/***************************** Include Files ************************/
#include "xparameters.h"
#include <stdio.h>
#include "xtmrctr.h"
#include "xintc.h"
#include "platform.h"
#include "xil_exception.h"
/********************** Constant Definitions ***********************/
/*
* The following constants map to the XPAR parameters created in the
* xparameters.h file. They are only defined here such that a user can
* easily change all the needed parameters in one place.
*/
#define TMRCTR_DEVICE_IDXPAR_TMRCTR_0_DEVICE_ID
#define INTC_DEVICE_IDXPAR_INTC_0_DEVICE_ID
#define TMRCTR_INTERRUPT_IDXPAR_INTC_0_TMRCTR_0_VEC_ID
/*
* The following constant determines which timer counter of the device
* that is used for this example, there are currently 2 timer counters
* in a device and this example uses the first one, 0, the timer numbers
* are 0 based
* */


#define TIMER_CNTR_0 0
/*
* The following constant is used to set the reset value of the timer
* counter, making this number larger reduces the amount of time this
* example consumes because it is the value the timer counter is loaded
* with when it is started
*/
#define RESET_VALUE 0xF0000000
/********************* Function Prototypes *************************/
int TmrCtrIntrExample(XIntc* IntcInstancePtr,XTmrCtr* InstancePtr,u16 DeviceId,u16 IntrId,u8 TmrCtrNumber);
void TimerCounterHandler(void *CallBackRef, u8 TmrCtrNumber);
/********************** Variable Definitions ************************/
XIntc InterruptController; /* The instance of the Interrupt Controller
*/
XTmrCtr TimerCounterInst; /* The instance of the Timer Counter */
/*
* The following variables are shared between non-interrupt processing
* and interrupt processing such that they must be global.
*/
volatile int TimerExpired;
/********************************************************************/
/**
* This function is the main function of the Tmrctr example using
* Interrupts.
*
* @paramNone.
*
* @returnXST_SUCCESS to indicate success, else XST_FAILURE to indicate
* a Failure.
*
* @noteNone.
*
*********************************************************************/
int main(void)
{
int Status;
/*
* Run the Timer Counter - Interrupt example.
*/
init_platform();

    print("Hello World\n\r");

// the parameter's name differ from those in the instruction, they are from the #include "xparameters.h" file
Status = TmrCtrIntrExample(&InterruptController,&TimerCounterInst,XPAR_TMRCTR_0_DEVICE_ID,XPAR_MICROBLAZE_0_AXI_INTC_AXI_TIMER_0_INTERRUPT_INTR,TIMER_CNTR_0); 
if (Status != XST_SUCCESS) {
	print("fail");
	return XST_FAILURE;
}
print("success");
return XST_SUCCESS;

}
/********************************************************************/
/**
* This function does a minimal test on the timer counter device and
* driver as a design example. The purpose of this function is to
* illustrate how to use the XTmrCtr component. It initializes a timer
* counter and then sets it up in compare mode with auto reload such that
* a periodic interrupt is generated.
*
* This function uses interrupt driven mode of the timer counter.
*
* @paramIntcInstancePtr is a pointer to the Interrupt Controller
* driver Instance
* @paramTmrCtrInstancePtr is a pointer to the XTmrCtr driver Instance
* @paramDeviceId is the XPAR_<TmrCtr_instance>_DEVICE_ID value from
* xparameters.h
* @paramIntrId is
XPAR_<INTC_instance>_<TmrCtr_instance>_INTERRUPT_INTR
* value from xparameters.h
* @paramTmrCtrNumber is the number of the timer to which this
* handler is associated with.
*
* @returnXST_SUCCESS if the Test is successful, otherwise XST_FAILURE
*
* @noteThis function contains an infinite loop such that if interrupts
* are not working it may never return.
*
*********************************************************************/
int TmrCtrIntrExample(XIntc* IntcInstancePtr,XTmrCtr* TmrCtrInstancePtr,u16 DeviceId,u16 IntrId,u8 TmrCtrNumber)
{
int Status;
int LastTimerExpired = 0;
/*
* Initialize the timer counter so that it's ready to use,
* specify the device ID that is generated in xparameters.h
*/
Status = XTmrCtr_Initialize(TmrCtrInstancePtr, DeviceId);
if (Status != XST_SUCCESS) {
return XST_FAILURE;
}
/*
* Initialize the interrupt controller driver so that
* it's ready to use, specify the device ID that is generated in
* xparameters.h
* */
Status = XIntc_Initialize(IntcInstancePtr, XPAR_MICROBLAZE_0_AXI_INTC_DEVICE_ID);
if (Status != XST_SUCCESS) {
return XST_FAILURE;
}
/*
* Connect a device driver handler that will be called when an
* interrupt for the device occurs, the device driver handler performs
* the specific interrupt processing for the device
*/
Status = XIntc_Connect(IntcInstancePtr, IntrId,
(XInterruptHandler)XTmrCtr_InterruptHandler,
(void *)TmrCtrInstancePtr);
if (Status != XST_SUCCESS) {
return XST_FAILURE;
}
/*
* Start the interrupt controller such that interrupts are enabled for
* all devices that cause interrupts, specific real mode so that
* the timer counter can cause interrupts thru the interrupt
* controller.
*/
Status = XIntc_Start(IntcInstancePtr, XIN_REAL_MODE);
if (Status != XST_SUCCESS) {
return XST_FAILURE;
}
/*
* Enable the interrupt for the timer counter
*/
XIntc_Enable(IntcInstancePtr, IntrId);
/*
* Initialize the exception table.
*/
Xil_ExceptionInit();
/*
* Register the interrupt controller handler with the exception table.
*/
Xil_ExceptionRegisterHandler(XIL_EXCEPTION_ID_INT,
(Xil_ExceptionHandler)
XIntc_InterruptHandler,
IntcInstancePtr);
/*
* Enable exceptions.
*/
Xil_ExceptionEnable();
if (Status != XST_SUCCESS) {
return XST_FAILURE;
}
/*
* Setup the handler for the timer counter that will be called from the
* interrupt context when the timer expires, specify a pointer to the
* * timer counter driver instance as the callback reference so the
* handler is able to access the instance data
*/
XTmrCtr_SetHandler(TmrCtrInstancePtr,
TimerCounterHandler,
TmrCtrInstancePtr);
/*
* Enable the interrupt of the timer counter so interrupts will occur
* and use auto reload mode such that the timer counter will reload
* itself automatically and continue repeatedly, without this option
* it would expire once only
*/
XTmrCtr_SetOptions(TmrCtrInstancePtr, TmrCtrNumber,
XTC_INT_MODE_OPTION | XTC_AUTO_RELOAD_OPTION);
/*
* Set a reset value for the timer counter such that it will expire
* earlier than letting it roll over from 0, the reset value is loaded
* into the timer counter when it is started
*/
XTmrCtr_SetResetValue(TmrCtrInstancePtr, TmrCtrNumber, RESET_VALUE);
/*
* Start the timer counter such that it's incrementing by default,
* then wait for it to timeout a number of times
*/
XTmrCtr_Start(TmrCtrInstancePtr, TmrCtrNumber);
while (1) {
/*
* Wait for the first timer counter to expire as indicated by the
* shared variable which the handler will increment
*/
while (TimerExpired == LastTimerExpired) {
}
LastTimerExpired = TimerExpired;
/*
* If it has expired a number of times, then stop the timer counter
* and stop this example
*/
if (TimerExpired == 1) {
XTmrCtr_Stop(TmrCtrInstancePtr, TmrCtrNumber);
break;
}
}
/*
* Disable the interrupt for the timer counter
*/
XIntc_Disable(IntcInstancePtr, DeviceId);
return XST_SUCCESS;
}
/********************************************************************/
/**
 * * This function is the handler which performs processing for the timer
* counter. It is called from an interrupt context such that the amount
* of processing performed should be minimized. It is called when the
* timer counter expires if interrupts are enabled.
*
* This handler provides an example of how to handle timer counter
* interrupts but is application specific.
*
* @paramCallBackRef is a pointer to the callback function
* @paramTmrCtrNumber is the number of the timer to which this
* handler is associated with.
*
* @returnNone.
*
* @noteNone.
*
*********************************************************************/
void TimerCounterHandler(void *CallBackRef, u8 TmrCtrNumber)
{
XTmrCtr *InstancePtr = (XTmrCtr *)CallBackRef;
/*
* Check if the timer counter has expired, checking is not necessary
* since that's the reason this function is executed, this just shows
* how the callback reference can be used as a pointer to the instance
* of the timer counter that expired, increment a shared variable so
* the main thread of execution can see the timer expired
*/
if (XTmrCtr_IsExpired(InstancePtr, TmrCtrNumber)) {
TimerExpired++;
  //this is where you can set how many times you want the timer to count,if this value is less than the value set in 
  //XTmrCtr_Start(TmrCtrInstancePtr, TmrCtrNumber), the program will get into the infinite loop
if(TimerExpired ==1 ) {                            
XTmrCtr_SetOptions(InstancePtr, TmrCtrNumber, 0);
}
}
}

